package MT::Plugin::EntryPrefsFixer;
use strict;

use MT;
our $VERSION = '0.3';

use MT::Plugin;
use base qw( MT::Plugin );

@MT::Plugin::EntryPrefsFixer::ISA = qw( MT::Plugin );

my $plugin = new MT::Plugin::EntryPrefsFixer( {
    id   => 'EntryPrefsFixer',
    key  => 'entryprefsfixer',
    name => 'EntryPrefsFixer',
    description => '<__trans phrase="When the entry prefs is preserved, the setting is applied to all users.">',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    l10n_class  => 'EntryPrefsFixer::L10N',
    version  => $VERSION,
    settings => new MT::PluginSettings( [
        [ 'entry_prefs', { Default => '' } ],
        [ 'page_prefs' , { Default => '' } ],
    ] ),
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        callbacks => {
            'MT::Permission::post_save'
                => \&_set_entry_prefs,
            'MT::App::CMS::template_source.edit_entry'
                => \&_hide_entry_prefs,
        },
   } );
}

sub _set_entry_prefs {
    my ( $cb, $obj ) = @_;
    my $app = MT->instance();
    if ( ref $app eq 'MT::App::CMS' ) {
        if ( _is_admin() ) {
            if ( $app->mode eq 'save_entry_prefs' ) {
                my $_type = $app->param( '_type' );
                require MT::Permission;
                my @perms = MT::Permission->load( { blog_id => $obj->blog_id,
                                                    author_id => { not => $obj->author_id } } );
                for my $p ( @perms ) {
                    if ( $_type eq 'page' ) {
                        $p->page_prefs( $obj->page_prefs );
                    } else {
                        $p->entry_prefs( $obj->entry_prefs );
                    }
                    $p->save or die $p->errstr;
                }
                if ( $_type eq 'page' ) {
                    $plugin->set_config_value( 'page_prefs', $obj->page_prefs ,'blog:' . $obj->blog_id );
                } else {
                    $plugin->set_config_value( 'entry_prefs', $obj->entry_prefs ,'blog:' . $obj->blog_id );
                }
            }
            if ( $obj->blog_id ) {
                my $permissions = $obj->permissions;
                require MT::Request;
                my $r = MT::Request->instance;
                return if $r->cache( 'cache_permission:' . $obj->id );
                $r->cache( 'cache_permission:' . $obj->id, 1 );
                if ( ( $permissions !~ /\'administer_website\'/ ) &&
                    ( $permissions !~ /\'administer_blog\'/ ) ) {
                    my $page_prefs = $plugin->get_config_value( 'page_prefs', 'blog:' . $obj->blog_id );
                    my $entry_prefs = $plugin->get_config_value( 'entry_prefs', 'blog:' . $obj->blog_id );
                    my $do;
                    if ( $page_prefs ) {
                        if ( $obj->page_prefs ne $page_prefs ) {
                            $obj->page_prefs( $page_prefs );
                            $do = 1;
                        }
                    }
                    if ( $entry_prefs ) {
                        if ( $obj->entry_prefs ne $entry_prefs ) {
                            $obj->entry_prefs( $entry_prefs );
                            $do = 1;
                        }
                    }
                    if ( $do ) {
                        $obj->save or die $obj->errstr;
                    }
                }
            }
        }
    }
}

sub _hide_entry_prefs {
    my ( $cb, $app, $tmpl ) = @_;
    if (! _is_admin() ) {
        my $show_display_options = quotemeta( '<$mt:setvar name="show_display_options_link" value="1"$>' );
        $$tmpl =~ s/$show_display_options//s;
        my $css =<<CSS;
<mt:setvarblock name="html_head" append="1">
<style type="text/css">
#display-options-widget {display:none;}
</style>
</mt:setvarblock>
CSS
        $$tmpl = $css . $$tmpl;
    }
}

sub _is_admin {
    my $app = MT->instance();
    my $perm = $app->user->is_superuser;
    if (! $perm ) {
        if ( $app->blog ) {
            my $class;
            if ( MT->version_number < 5 ) {
                $class = 'blog';
            } else {
                $class = $app->blog->class;
            }
            my $permission = 'can_administer_' . $class;
            $perm = $app->user->permissions( $app->blog->id )->$permission;
        }
    }
    return $perm;
}

1;